@@ -412,7 +412,7 @@ sqlite_st_execute (SV *sth, imp_sth_t *imp_sth)
         /* warn("Finalize\n"); */
         sqlite3_reset(imp_sth->stmt);
         imp_sth->nrow = sqlite3_changes(imp_dbh->db);
-        DBIc_ACTIVE_on(imp_sth);
+        /* DBIc_ACTIVE_on(imp_sth); */
         /* warn("Total changes: %d\n", sqlite3_total_changes(imp_dbh->db)); */
         /* warn("Nrow: %d\n", imp_sth->nrow); */
         return imp_sth->nrow;
@@ -1102,4 +1102,149 @@ sqlite3_db_create_aggregate( SV *dbh, const char *name, int argc, SV *aggr_pkg )
     }
 }
 
+
+int sqlite_db_collation_dispatcher(void *func, int len1, const void *string1,
+                                               int len2, const void *string2)
+{
+    dSP;
+    int cmp;
+    int n_retval;
+
+    ENTER;
+    SAVETMPS;
+    PUSHMARK(SP);
+    XPUSHs( sv_2mortal ( newSVpvn( string1, len1) ) );
+    XPUSHs( sv_2mortal ( newSVpvn( string2, len2) ) );
+    PUTBACK;
+    n_retval = call_sv((void*)func, G_SCALAR);
+    if (n_retval != 1) {
+      croak("collation function returned %d arguments", n_retval);
+    }
+    SPAGAIN;
+    cmp = POPi;
+    PUTBACK;
+    FREETMPS;
+    LEAVE;
+
+    return cmp;
+}
+
+int sqlite_db_collation_dispatcher_utf8(
+  void *func, int len1, const void *string1,
+              int len2, const void *string2)
+{
+    dSP;
+    int cmp;
+    int n_retval;
+    SV *sv1, *sv2;
+
+    ENTER;
+    SAVETMPS;
+    PUSHMARK(SP);
+    sv1 = newSVpvn( string1, len1);
+    SvUTF8_on(sv1);
+    sv2 = newSVpvn( string2, len2);
+    SvUTF8_on(sv2);
+    XPUSHs( sv_2mortal ( sv1 ) );
+    XPUSHs( sv_2mortal ( sv2 ) );
+    PUTBACK;
+    n_retval = call_sv((void*)func, G_SCALAR);
+    if (n_retval != 1) {
+      croak("collation function returned %d arguments", n_retval);
+    }
+    SPAGAIN;
+    cmp = POPi;
+    PUTBACK;
+    FREETMPS;
+    LEAVE;
+
+    return cmp;
+}
+
+
+void
+sqlite3_db_create_collation( SV *dbh, const char *name, SV *func )
+{
+    D_imp_dbh(dbh);
+    int rv, rv2;
+    void *aa = "aa";
+    void *zz = "zz";
+
+    SV *func_sv = newSVsv(func);
+
+    /* Check that this is a proper collation function */
+    rv = sqlite_db_collation_dispatcher(func_sv, 2, aa, 2, aa);
+    if (rv != 0) {
+      warn("improper collation function: %s(aa, aa) returns %d!", name, rv); 
+    }
+    rv  = sqlite_db_collation_dispatcher(func_sv, 2, aa, 2, zz);
+    rv2 = sqlite_db_collation_dispatcher(func_sv, 2, zz, 2, aa);
+    if (rv2 != (rv * -1)) {
+      warn("improper collation function: '%s' is not symmetric", name); 
+    }
+
+    /* Copy the func reference so that it can be deallocated at disconnect */
+    av_push( imp_dbh->functions, func_sv );
+
+    /* Register the func within sqlite3 */
+    rv = sqlite3_create_collation( 
+      imp_dbh->db, name, SQLITE_UTF8,
+      func_sv, 
+      imp_dbh->unicode ? sqlite_db_collation_dispatcher_utf8 
+                       : sqlite_db_collation_dispatcher
+      );
+
+    if ( rv != SQLITE_OK )
+    {
+        croak( "sqlite_create_collation failed with error %s", 
+               sqlite3_errmsg(imp_dbh->db) );
+    }
+}
+
+
+int sqlite_db_progress_handler_dispatcher( void *handler )
+{
+    dSP;
+    int n_retval;
+    int retval;
+
+    PUSHMARK(SP);
+    n_retval = call_sv( handler, G_SCALAR );
+    if ( n_retval != 1 ) {
+      croak( "progress_handler returned %d arguments", n_retval );
+    }
+    SPAGAIN;
+    retval = POPi;
+    PUTBACK;
+
+    return retval;
+}
+
+
+
+void
+sqlite3_db_progress_handler( SV *dbh, int n_opcodes, SV *handler )
+{
+    D_imp_dbh(dbh);
+
+    if (handler == &PL_sv_undef) {
+      /* remove previous handler */
+      sqlite3_progress_handler( imp_dbh->db, 0, NULL, NULL);
+    }
+    else {
+      int rv;
+      SV *handler_sv = newSVsv(handler);
+
+      /* Copy the handler ref so that it can be deallocated at disconnect */
+      av_push( imp_dbh->functions, handler_sv );
+
+      /* Register the func within sqlite3 */
+      sqlite3_progress_handler( imp_dbh->db, n_opcodes, 
+                                sqlite_db_progress_handler_dispatcher,
+                                handler_sv );
+    }
+}
+
+
+
 /* end */
